// Squawk messenger. 
// Copyright (C) 2019  Yury Gubich <blue@macaw.me>
// 
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.

#ifndef DATABASE_CACHE_H
#define DATABASE_CACHE_H

#include "database.h"
#include "table.h"
#include <map>
#include <set>

template <class K, class V>
class DataBase::Cache : public DataBase::Table<K, V> {
    friend class DataBase;
    enum class Mode {
        nothing,
        size,
        full
    };
protected:
    Cache(const std::string& name, DataBase* parent);
    ~Cache() override;

public:
    virtual void addRecord(const K& key, const V& value) override;
    virtual void changeRecord(const K& key, const V& value) override;
    virtual void removeRecord(const K& key) override;
    virtual V getRecord(const K& key) const override;
    //virtual uint32_t count() const;
    //virtual void drop();

protected:
    Mode mode;
    std::map<K, V>* cache;
    std::set<K>* abscent;
};

#include "cache.hpp"

#endif // DATABASE_CACHE_H
